use std::cmp::PartialEq;
use std::collections::HashSet;
use std::convert::TryFrom;
use std::error::Error;

use rusqlite::Row;
use serde_derive::{Deserialize, Serialize};

use crate::id::Id;
use crate::world::{Area, AreaType};

#[derive(Debug, Deserialize, Serialize)]
pub struct Zone {
	pub id: Id,
	pub parent: Id,
	pub name: String,
	pub users_visible: bool,
	pub areas: HashSet<Id>,
}

impl Zone {
	/// Create a new `Zone` from a `&Zone`.
	///
	/// # Arguments
	///
	/// * `zone: &Zone` : Reference to a `Zone`
	///
	/// # Examples
	///
	/// ```
	/// use std::collections::HashSet;
	/// use rude::id::Id;
	/// use rude::world::Zone;
	///
	/// let zone = Zone {
	///     id: Id::new(),
	///     parent: Id::new(),
	///     name: "Zone".into(),
	///     users_visible: false,
	///     areas: HashSet::new(),
	/// };
	///
	/// assert_eq!(zone, Zone::from_ref(&zone));
	/// ```
	pub fn from_ref(zone: &Self) -> Self {
		Self {
			id: zone.id,
			parent: zone.parent,
			name: zone.name.clone(),
			users_visible: zone.users_visible,
			areas: zone.areas.clone(),
		}
	}
}

impl PartialEq for Zone {
	fn eq(&self, other: &Self) -> bool {
		self.id == other.id
			&& self.parent == other.parent
			&& self.name == other.name
			&& self.users_visible == other.users_visible
			&& self.areas == other.areas
	}
}

impl Area for Zone {
	fn id(&self) -> Id {
		self.id
	}

	fn parent(&self) -> Id {
		self.parent
	}

	fn name(&self) -> String {
		self.name.to_owned()
	}

	fn visible(&self) -> bool {
		self.users_visible
	}

	fn area_type(&self) -> AreaType {
		AreaType::Zone
	}
}

impl<'a> TryFrom<&Row<'a>> for Zone {
	type Error = Box<dyn Error>;

	fn try_from(row: &Row) -> Result<Self, Self::Error> {
		Ok(Self {
			id: row.get("id")?,
			parent: row.get("parent")?,
			name: row.get("name")?,
			users_visible: row.get("users_visible")?,
			areas: HashSet::new(),
		})
	}
}
